library(ggplot2)
library(gridExtra)
library(ssh)
setwd("/.../")


####################################################################
####################### DATA PREPARATION ###########################
####################################################################

rawResults<-read.csv(file="results_GS_NES.csv",header=TRUE,sep=";",stringsAsFactor=FALSE)
pleda<-read.csv(file="results_EDA.csv",header=TRUE,sep=";",stringsAsFactor=FALSE)
colnames(rawResults)<-c("Instance","Repetition","Algorithm","Lambda","Alpha","Fitness","EvalsNeeded","Time")
colnames(pleda)<-c("Instance","Repetition","Algorithm","Fitness","Time")
rawResults$Lambda <- NULL
rawResults$Alpha <- NULL
rawResults$EvalsNeeded<-NULL
rawResults<- rbind(rawResults,pleda)
rawResults$Algorithm[rawResults$Algorithm=="PLNSA"]<-"NES*"
rawResults$Algorithm[rawResults$Algorithm=="PLN"]<-"NES"
rawResults$Algorithm[rawResults$Algorithm=="PL"]<-"GS"
rawResults$Algorithm[rawResults$Algorithm=="PLSA"]<-"GS*"
rawResults$Algorithm[rawResults$Algorithm=="PLEDA"]<-"EDA"
results <- na.omit(rawResults)


results_bks<-read.csv(file="best_known_results.csv",header=TRUE,sep=";",stringsAsFactor=FALSE)

Size=seq(1,length(results$Instance),1)
Normalized=seq(1,length(results$Instance),1)
results<-cbind(results,Size)
results<-cbind(results,Normalized)
#Performance results comparison
colnames(results)<-c("Instance","Repetition","Algorithm","Fitness","Time", "Size", "Normalized")
for (i in 1:length(results$Instance)) 
{
  best_known<-results_bks[results_bks$Instance==results$Instance[i],]
  best_fit<-as.numeric(as.character(best_known$Result))
  fitness<-results[i,]$Fitness
  results[i,]$Normalized<-abs(fitness-best_fit)/best_fit
  results[i,]$Size<-results_bks[results_bks$Instance==results$Instance[i],]$Size
}
results$Repetition<-as.integer(results$Repetition)
results$Size<-as.integer(results$Size)

################################################################################################
###################  ALGORITHM-INSTANCE PAIR BAYESIAN ANALYSIS #################################
################################################################################################
library("scmamp")
getCredibleIntervalsWeights <- function(posterior.samples, interval.size=0.9) {
  qmin <- (1-interval.size)/2
  qmax <- 1-qmin
  lower.bound <- apply(posterior.samples, MARGIN=2, FUN=quantile, p=qmin) 
  upper.bound <- apply(posterior.samples, MARGIN=2, FUN=quantile, p=qmax) 
  expectation <- apply(posterior.samples, MARGIN=2, FUN=mean) 
  return (data.frame(Expected=expectation, Lower_bound=lower.bound, Upper_bound=upper.bound))
}

repetitions<-30
algorithms<-3
chains<-20
################ INDIVIDUAL BAYESIAN PLOTS IO BENCHMARK ###################
data<-results[grep("_", results$Instance, invert = TRUE),] ### FILTER IO BENCHMARK INSTANCES
instances<-unique(data$Instance)

plots<-list()
for (instance in instances){
  ##instance<-"N-be75eec"
  cat("Instance: ",instance, "\n\n")
  # Bayesian analysis per algorithm instance pair
  dataBayesian <- data.frame(matrix(NA, nrow = repetitions, ncol = algorithms)) 
  colnames(dataBayesian)<-c("GS","NES","EDA")
  
      dataBayesian$GS<-data[(data$Instance==instance & data$Algorithm=="GS"),]$Fitness[1:repetitions] # GS
      dataBayesian$NES<-data[(data$Instance==instance & data$Algorithm=="NES"),]$Fitness[1:repetitions] # NES
      dataBayesian$EDA<-data[(data$Instance==instance & data$Algorithm=="EDA"),]$Fitness[1:repetitions] # EDA
      
      # NA in a row... then remove row. Becareful with this row!!! You might remove lots of lines...
      dataBayesian3 <- na.omit(dataBayesian)

  p<-NULL
  if(nrow(dataBayesian3)==repetitions){
    ## Statistical Analysis
    pl_model <-  bPlackettLuceModel(x.matrix=dataBayesian3, min=FALSE, nsim=4000, nchains=chains, parallel=TRUE,)
    processed.results <- getCredibleIntervalsWeights(pl_model$posterior.weights, interval.size=0.9)
    dataBayesian2 <- data.frame(Algorithm=rownames(processed.results), processed.results)
    p<-ggplot(dataBayesian2, aes(y=Expected, ymin=Lower_bound, ymax=Upper_bound, x=Algorithm)) + geom_errorbar() + geom_point(col="darkgreen", size=2) +  theme_bw() + coord_flip() + labs(y="Probability of winning")+ ggtitle(label = instance)
  }
  else{
    message(paste("     WARNING! No enough or missing repetitions for instance: ",instance))
    sumGS<-sum(is.na(dataBayesian$GS))
    if (sumGS!=0){
      message(paste("        GS ",sumGS," repetitions missing."))
      fileConn<-file("errors.txt","a")
      writeLines(paste(instance," GS ",sumGS," repetitions missing."), fileConn)
      close(fileConn)
    }
    sumNES<-sum(is.na(dataBayesian$NES))
    if (sumNES!=0){
      message(paste("        NES ",sumNES," repetitions missing."))
      fileConn<-file("errors.txt","a")
      writeLines(paste(instance," NES ",sumNES," repetitions missing."), fileConn)
      close(fileConn)
    }
    sumEDA<-sum(is.na(dataBayesian$EDA))
    if (sumEDA!=0){
      message(paste("        EDA ",sumEDA," repetitions missing."))
      fileConn<-file("errors.txt","a")
      writeLines(paste(instance," EDA ",sumEDA," repetitions missing."), fileConn)
      close(fileConn)
    }
  }
  plots[[instance]]<-p
}
#names(plots)<-instances
  
grid_mode<-grid.arrange(grobs=plots, ncol=5, top="Bayesian Performance Analysis per Instance from IO benchmark")
ggsave("IndividualPerformanceBayesian_IO.png",grid_mode, width = 360, height = 480, dpi = 300, units = "mm", device='png')

################ INDIVIDUAL BAYESIAN PLOTS xLOLIB - 150 BENCHMARK ###################
data<-results[grep("_150", results$Instance, invert = FALSE),] ### FILTER XLOLIB - 150 BENCHMARK INSTANCES
instances<-unique(data$Instance)

plots<-list()
for (instance in instances){

  # Bayesian analysis per algorithm instance pair
  ##instance<-"N-be75eec"
  cat("Instance: ",instance, "\n\n")
  # Bayesian analysis per algorithm instance pair
  dataBayesian <- data.frame(matrix(NA, nrow = repetitions, ncol = algorithms)) 
  colnames(dataBayesian)<-c("GS","NES","EDA")
  
  dataBayesian$GS<-data[(data$Instance==instance & data$Algorithm=="GS"),]$Fitness[1:repetitions] # GS
  dataBayesian$NES<-data[(data$Instance==instance & data$Algorithm=="NES"),]$Fitness[1:repetitions] # NES
  dataBayesian$EDA<-data[(data$Instance==instance & data$Algorithm=="EDA"),]$Fitness[1:repetitions] # EDA
  
  # NA in a row... then remove row. Becareful with this row!!! You might remove lots of lines...
  dataBayesian3 <- na.omit(dataBayesian)
  
  p<-NULL
  if(nrow(dataBayesian3)==repetitions){
    ## Statistical Analysis
    pl_model <-  bPlackettLuceModel(x.matrix=dataBayesian3, min=FALSE, nsim=4000, nchains=chains, parallel=TRUE)
    processed.results <- getCredibleIntervalsWeights(pl_model$posterior.weights, interval.size=0.9)
    dataBayesian2 <- data.frame(Algorithm=rownames(processed.results), processed.results)
  
    p<-ggplot(dataBayesian2, aes(y=Expected, ymin=Lower_bound, ymax=Upper_bound, x=Algorithm)) + geom_errorbar() + geom_point(col="darkgreen", size=2) +  theme_bw() + coord_flip() + labs(y="Probability of winning")+ ggtitle(label = instance)
  }
  else{
    message(paste("     WARNING! No enough or missing repetitions for instance: ",instance))
    sumGS<-sum(is.na(dataBayesian$GS))
    if (sumGS!=0){
      message(paste("        GS ",sumGS," repetitions missing."))
      fileConn<-file("errors.txt","a")
      writeLines(paste(instance," GS ",sumGS," repetitions missing."), fileConn)
      close(fileConn)
    }
    sumNES<-sum(is.na(dataBayesian$NES))
    if (sumNES!=0){
      message(paste("        NES ",sumNES," repetitions missing."))
      fileConn<-file("errors.txt","a")
      writeLines(paste(instance," NES ",sumNES," repetitions missing."), fileConn)
      close(fileConn)
    }
    sumEDA<-sum(is.na(dataBayesian$EDA))
    if (sumEDA!=0){
      message(paste("        EDA ",sumEDA," repetitions missing."))
      fileConn<-file("errors.txt","a")
      writeLines(paste(instance," EDA ",sumEDA," repetitions missing."), fileConn)
      close(fileConn)
    }
  }
  plots[[instance]]<-p
}
#names(plots)<-instances


grid_mode<-grid.arrange(grobs=plots, ncol=5, top="Bayesian Performance Analysis per Instance from xLOLIB - 150 benchmark")
ggsave("IndividualPerformanceBayesian_150.png",grid_mode, width = 360, height = 480, dpi = 300, units = "mm", device='png')

################ INDIVIDUAL BAYESIAN PLOTS xLOLIB - 250 BENCHMARK ###################
data<-results[grep("_250", results$Instance, invert = FALSE),] ### FILTER XLOLIB - 250 BENCHMARK INSTANCES
instances<-unique(data$Instance)

plots<-list()
for (instance in instances){
  # Bayesian analysis per algorithm instance pair
  ##instance<-"N-be75eec"
  cat("Instance: ",instance, "\n\n")
  # Bayesian analysis per algorithm instance pair
  dataBayesian <- data.frame(matrix(NA, nrow = repetitions, ncol = algorithms)) 
  colnames(dataBayesian)<-c("GS","NES","EDA")
  
  dataBayesian$GS<-data[(data$Instance==instance & data$Algorithm=="GS"),]$Fitness[1:repetitions] # GS
  dataBayesian$NES<-data[(data$Instance==instance & data$Algorithm=="NES"),]$Fitness[1:repetitions] # NES
  dataBayesian$EDA<-data[(data$Instance==instance & data$Algorithm=="EDA"),]$Fitness[1:repetitions] # EDA
  
  # NA in a row... then remove row. Becareful with this row!!! You might remove lots of lines...
  dataBayesian3 <- na.omit(dataBayesian)
  
  p<-NULL
  if(nrow(dataBayesian3)==repetitions){
    ## Statistical Analysis
    pl_model <-  bPlackettLuceModel(x.matrix=dataBayesian3, min=FALSE, nsim=4000, nchains=chains, parallel=TRUE)
    processed.results <- getCredibleIntervalsWeights(pl_model$posterior.weights, interval.size=0.9)
    dataBayesian2 <- data.frame(Algorithm=rownames(processed.results), processed.results)
  
    p<-ggplot(dataBayesian2, aes(y=Expected, ymin=Lower_bound, ymax=Upper_bound, x=Algorithm)) + geom_errorbar() + geom_point(col="darkgreen", size=2) +  theme_bw() + coord_flip() + labs(y="Probability of winning")+ ggtitle(label = instance)
  }
  else{
    message(paste("     WARNING! No enough or missing repetitions for instance: ",instance))
    sumGS<-sum(is.na(dataBayesian$GS))
    if (sumGS!=0){
      message(paste("        GS ",sumGS," repetitions missing."))
      fileConn<-file("errors.txt","a")
      writeLines(paste(instance," GS ",sumGS," repetitions missing."), fileConn)
      close(fileConn)
    }
    sumNES<-sum(is.na(dataBayesian$NES))
    if (sumNES!=0){
      message(paste("        NES ",sumNES," repetitions missing."))
      fileConn<-file("errors.txt","a")
      writeLines(paste(instance," NES ",sumNES," repetitions missing."), fileConn)
      close(fileConn)
    }
    sumEDA<-sum(is.na(dataBayesian$EDA))
    if (sumEDA!=0){
      message(paste("        EDA ",sumEDA," repetitions missing."))
      fileConn<-file("errors.txt","a")
      writeLines(paste(instance," EDA ",sumEDA," repetitions missing."), fileConn)
      close(fileConn)
    }
  }
  plots[[instance]]<-p
}
#names(plots)<-instances

grid_mode<-grid.arrange(grobs=plots, ncol=5, top="Bayesian Performance Analysis per Instance from xLOLIB - 250 benchmark")
ggsave("IndividualPerformanceBayesian_250.png",grid_mode, width = 360, height = 480, dpi = 300, units = "mm", device='png')


######################################################################################
###################  AGGREGATED POST-PROCESSING ######################################
######################################################################################

########################## AGGREGATE RESULTS AS MEDIAN VALUES ##############################
data_avg<-aggregate(results[, c(4,5,7)], list(results$Instance, results$Algorithm), median)
colnames(data_avg)<-c("Instance","Algorithm","Fitness","Time", "Norm")

################ WRITE RESULTS IN FILES FOR LOADING IN TABLES IN APPENDIX ###################
data_avg <- data_avg[order(data_avg$Algorithm,data_avg$Instance),]
write.csv(data_avg[grep("_", data_avg$Instance, invert = TRUE), ],file="postprocessed_results_IO.csv")
write.csv(data_avg[grep("_150", data_avg$Instance, invert = FALSE), ],file="postprocessed_results_xLOLIB_150.csv")
write.csv(data_avg[grep("_250", data_avg$Instance, invert = FALSE), ],file="postprocessed_results_xLOLIB_250.csv")


########################### BAYESIAN ANALYSIS BY BENCHMARKS #########################################

dataIO<-data_avg[grep("_", data_avg$Instance, invert = TRUE),] ### FILTER IO BENCHMARK INSTANCES
dataxLOLIB_150<-data_avg[grep("_150", data_avg$Instance, invert = FALSE),] ### FILTER XLOLIB - 150 BENCHMARK INSTANCES
dataxLOLIB_250<-data_avg[grep("_250", data_avg$Instance, invert = FALSE),] ### FILTER XLOLIB - 250 BENCHMARK INSTANCES

plots<-list()
algorithms<-3

############################## Credible intervals for IO benchmark
instances<-unique(dataIO$Instance)
dataBayesian <- data.frame(matrix(NA, nrow = length(instances), ncol = algorithms)) 
colnames(dataBayesian)<-c("GS","NES","EDA")  
dat<-dataIO[(dataIO$Algorithm=="GS"),]$Fitness
dataBayesian$GS[1:length(dat)]<-dat # GS
dat<-dataIO[(dataIO$Algorithm=="NES"),]$Fitness
dataBayesian$NES[1:length(dat)]<-dat # NES
dat<-dataIO[(dataIO$Algorithm=="EDA"),]$Fitness
dataBayesian$EDA[1:length(dat)]<-dat # EDA

# NA in a row... then remove row. Becareful with this row!!! You might remove lots of lines...
dataBayesian <- na.omit(dataBayesian)

## Statistical Analysis
pl_model <-  bPlackettLuceModel(x.matrix=dataBayesian, min=FALSE, nsim=4000, nchains=20, parallel=TRUE)
processed.results <- getCredibleIntervalsWeights(pl_model$posterior.weights, interval.size=0.9)
dataBayesian2 <- data.frame(Algorithm=rownames(processed.results), processed.results)
p<-ggplot(dataBayesian2, aes(y=Expected, ymin=Lower_bound, ymax=Upper_bound, x=Algorithm)) + geom_errorbar() + geom_point(col="darkgreen", size=2) +  theme_bw() + coord_flip() + labs(y="Probability of winning")+ ggtitle(label = "IO benchmark")
plots[["IO"]]<-p

############################## Credible intervals for xLOLIB benchmark - n=150
instances<-unique(dataxLOLIB_150$Instance)
dataBayesian <- data.frame(matrix(NA, nrow = length(instances), ncol = algorithms)) 
colnames(dataBayesian)<-c("GS","NES","EDA")  
dat<-dataxLOLIB_150[(dataxLOLIB_150$Algorithm=="GS"),]$Fitness
dataBayesian$GS[1:length(dat)]<-dat # GS
dat<-dataxLOLIB_150[(dataxLOLIB_150$Algorithm=="NES"),]$Fitness
dataBayesian$NES[1:length(dat)]<-dat # NES
dat<-dataxLOLIB_150[(dataxLOLIB_150$Algorithm=="EDA"),]$Fitness
dataBayesian$EDA[1:length(dat)]<-dat # EDA

# NA in a row... then remove row. Becareful with this row!!! You might remove lots of lines...
dataBayesian <- na.omit(dataBayesian)

## Statistical Analysis
pl_model <-  bPlackettLuceModel(x.matrix=dataBayesian, min=FALSE, nsim=4000, nchains=20, parallel=TRUE)
processed.results <- getCredibleIntervalsWeights(pl_model$posterior.weights, interval.size=0.9)
dataBayesian2 <- data.frame(Algorithm=rownames(processed.results), processed.results)
p<-ggplot(dataBayesian2, aes(y=Expected, ymin=Lower_bound, ymax=Upper_bound, x=Algorithm)) + geom_errorbar() + geom_point(col="darkgreen", size=2) +  theme_bw() + coord_flip() + labs(y="Probability of winning")+ ggtitle(label = "xLOLIB - n=150 benchmark")
plots[["xLOLIB150"]]<-p


############################## Credible intervals for xLOLIB benchmark - n=250
instances<-unique(dataxLOLIB_250$Instance)
dataBayesian <- data.frame(matrix(NA, nrow = length(instances), ncol = algorithms)) 
colnames(dataBayesian)<-c("GS","NES","EDA")
dat<-dataxLOLIB_250[(dataxLOLIB_250$Algorithm=="GS"),]$Fitness
dataBayesian$GS[1:length(dat)]<-dat # GS
dat<-dataxLOLIB_250[(dataxLOLIB_250$Algorithm=="NES"),]$Fitness
dataBayesian$NES[1:length(dat)]<-dat # NES
dat<-dataxLOLIB_250[(dataxLOLIB_250$Algorithm=="EDA"),]$Fitness
dataBayesian$EDA[1:length(dat)]<-dat # EDA

# NA in a row... then remove row. Becareful with this row!!! You might remove lots of lines...
dataBayesian <- na.omit(dataBayesian)

## Statistical Analysis
pl_model <-  bPlackettLuceModel(x.matrix=dataBayesian, min=FALSE, nsim=4000, nchains=20, parallel=TRUE)
processed.results <- getCredibleIntervalsWeights(pl_model$posterior.weights, interval.size=0.9)
dataBayesian2 <- data.frame(Algorithm=rownames(processed.results), processed.results)
p<-ggplot(dataBayesian2, aes(y=Expected, ymin=Lower_bound, ymax=Upper_bound, x=Algorithm)) + geom_errorbar() + geom_point(col="darkgreen", size=2) +  theme_bw() + coord_flip() + labs(y="Probability of winning")+ ggtitle(label = "xLOLIB - n=250 benchmark")
plots[["xLOLIB250"]]<-p

############################## Create plot withe the credible intervals.
names(plots)<-c("IO", "xLOLIB - n=150", "xLOLIB - n=250")

grid_mode<-grid.arrange(grobs=plots, ncol=3, top="Bayesian Performance Analysis per benchmark")
ggsave("BenchmarkPerformanceBayesian.png",grid_mode, width = 250, height = 75, dpi = 300, units = "mm", device='png')



